@@ -2,13 +2,13 @@
 """
     MoinMoin - UserPreferences Form and User Browser
 
-    @copyright: 2001-2004 by Jrgen Hermann <jh@web.de>
+    @copyright: 2001-2004 Juergen Hermann <jh@web.de>,
+                2003-2007 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
 
-import string, time, re
+import time
 from MoinMoin import user, util, wikiutil
-from MoinMoin.util import web, mail, timefuncs
 from MoinMoin.widget import html
 
 _debug = 0
@@ -38,10 +38,6 @@ class UserSettingsHandler:
 
         Each line is a page name, empty lines ignored.
 
-        Items can use '_' as spaces, needed by [name_with_spaces label]
-        format used in quicklinks. We do not touch those names here, the
-        underscores are handled later by the theme code.
-
         @param key: the form key to get
         @rtype: list of unicode strings
         @return: list of normalized names
@@ -53,79 +49,101 @@ class UserSettingsHandler:
             item = item.strip()
             if not item:
                 continue
-            # Normalize names - except [name_with_spaces label]
-            # Commented out to allow URLs
-            #if not (item.startswith('[') and item.endswith(']')):
-            #    item = self.request.normalizePagename(item)
             items.append(item)
         return items
 
     def handleData(self):
         _ = self._
         form = self.request.form
-    
+
         if form.has_key('cancel'):
             return
 
         if form.has_key('account_sendmail'):
             if not self.cfg.mail_enabled:
                 return _("""This wiki is not enabled for mail processing.
-Contact the owner of the wiki, who can enable email.""")
+Contact the owner of the wiki, who can enable email.""", formatted=False)
             try:
-                email = form['email'][0].lower()
+                email = wikiutil.clean_input(form['email'][0].lower())
+                if not email:
+                    raise KeyError # we raise KeyError if the string is empty
             except KeyError:
-                return _("Please provide a valid email address!")
-    
-            users = user.getUserList(self.request)
-            for uid in users:
-                theuser = user.User(self.request, uid)
-                if theuser.valid and theuser.email.lower() == email:
-                    msg = theuser.mailAccountData()
-                    return wikiutil.escape(msg)
+                try:
+                    username = wikiutil.clean_input(form['name'][0])
+                    if not username:
+                        raise KeyError
+                except KeyError:
+                    return _("Please provide a valid email address!", formatted=False)
 
-            return _("Found no account matching the given email address '%(email)s'!") % {'email': wikiutil.escape(email)}
+                u = user.User(self.request, user.getUserId(self.request, username))
+                if u.valid:
+                    is_ok, msg = u.mailAccountData()
+                    if not is_ok:
+                        return wikiutil.escape(msg)
+
+                return _("If an account with this username exists, an email was sent.", formatted=False)
+
+            u = user.get_by_email_address(self.request, email)
+            if u:
+                is_ok, msg = u.mailAccountData()
+                return wikiutil.escape(msg)
+
+            return _("Found no account matching the given email address '%(email)s'!", formatted=False) % {'email': wikiutil.escape(email)}
 
         if (form.has_key('create') or
             form.has_key('create_only') or
             form.has_key('create_and_mail')):
             if self.request.request_method != 'POST':
-                return _("Use UserPreferences to change your settings or create an account.")
+                return _("Use UserPreferences to change your settings or create an account.", formatted=False)
+            
+            from MoinMoin.security.textcha import TextCha
+            if not TextCha(self.request).check_answer_from_form():
+                return _('TextCha: Wrong answer! Go back and try again...', formatted=False)
+
             # Create user profile
             if form.has_key('create'):
                 theuser = self.request.get_user_from_form()
             else:
                 theuser = user.User(self.request, auth_method="request:152")
-                
+
             # Require non-empty name
             try:
                 theuser.name = form['name'][0]
             except KeyError:
-                return _("Empty user name. Please enter a user name.")
+                return _("Empty user name. Please enter a user name.", formatted=False)
 
             # Don't allow users with invalid names
             if not user.isValidName(self.request, theuser.name):
                 return _("""Invalid user name {{{'%s'}}}.
 Name may contain any Unicode alpha numeric character, with optional one
-space between words. Group page name is not allowed.""") % wikiutil.escape(theuser.name)
+space between words. Group page name is not allowed.""", formatted=True, percent=True) % wikiutil.escape(theuser.name)
 
             # Is this an existing user trying to change information or a new user?
             # Name required to be unique. Check if name belong to another user.
             newuser = 1
             if user.getUserId(self.request, theuser.name):
                 if theuser.name != self.request.user.name:
-                    return _("This user name already belongs to somebody else.")
+                    return _("This user name already belongs to somebody else.", formatted=False)
                 else:
                     newuser = 0
 
             # try to get the password and pw repeat
             password = form.get('password', [''])[0]
-            password2 = form.get('password2',[''])[0]
+            password2 = form.get('password2', [''])[0]
 
             # Check if password is given and matches with password repeat
             if password != password2:
-                return _("Passwords don't match!")
-            if not password and newuser:
-                return _("Please specify a password!")
+                return _("Passwords don't match!", formatted=False)
+
+            if newuser:
+                if not password:
+                    return _("Please specify a password!", formatted=False)
+                pw_checker = self.request.cfg.password_checker
+                if pw_checker:
+                    pw_error = pw_checker(theuser.name, password)
+                    if pw_error:
+                        return _("Password not acceptable: %s", formatted=False) % pw_error
+
             # Encode password
             if password and not password.startswith('{SHA}'):
                 try:
@@ -135,86 +153,98 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                     return "Can't encode password: %s" % str(err)
 
             # try to get the (required) email
-            email = form.get('email', [''])[0]
+            email = wikiutil.clean_input(form.get('email', [''])[0])
             theuser.email = email.strip()
             if not theuser.email:
                 return _("Please provide your email address. If you lose your"
-                         " login information, you can get it by email.")
+                         " login information, you can get it by email.", formatted=False)
 
             # Email should be unique - see also MoinMoin/script/accounts/moin_usercheck.py
             if theuser.email and self.request.cfg.user_email_unique:
-                users = user.getUserList(self.request)
-                for uid in users:
-                    if uid == theuser.id:
-                        continue
-                    thisuser = user.User(self.request, uid)
-                    if thisuser.email == theuser.email and not thisuser.disabled:
-                        return _("This email already belongs to somebody else.")
+                if user.get_by_email_address(self.request, theuser.email):
+                    return _("This email already belongs to somebody else.", formatted=False)
 
             # save data
             theuser.save()
             if form.has_key('create_and_mail'):
                 theuser.mailAccountData()
-            
-            result = _("User account created! You can use this account to login now...")
+
+            result = _("User account created! You can use this account to login now...", formatted=False)
             if _debug:
                 result = result + util.dumpFormData(form)
             return result
-        
-        
-        # Select user profile (su user) - only works with cookie auth active.
+
+
+        # Select user profile (su user) - only works with logged-in user session support
         if form.has_key('select_user'):
-            if (wikiutil.checkTicket(self.request.form['ticket'][0]) and
+            if (self.request.session and
+                wikiutil.checkTicket(self.request, self.request.form['ticket'][0]) and
                 self.request.request_method == 'POST' and
-                self.request.user.isSuperUser()):
+                (self.request.user.isSuperUser() or
+                 (not self.request._setuid_real_user is None
+                  and (self.request._setuid_real_user.isSuperUser())))):
                 su_user = form.get('selected_user', [''])[0]
                 uid = user.getUserId(self.request, su_user)
-                theuser = user.User(self.request, uid)
-                theuser.disabled = None
-                theuser.save()
-                from MoinMoin import auth
-                auth.setCookie(self.request, theuser)
-                self.request.user = theuser
-                return  _("Use UserPreferences to change settings of the selected user account")
+                if (not self.request._setuid_real_user is None
+                    and uid == self.request._setuid_real_user.id):
+                    del self.request.session['setuid']
+                    self.request.user = self.request._setuid_real_user
+                    self.request._setuid_real_user = None
+                else:
+                    theuser = user.User(self.request, uid)
+                    theuser.disabled = None
+                    self.request.session['setuid'] = uid
+                    self.request._setuid_real_user = self.request.user
+                    # now continue as the other user
+                    self.request.user = theuser
+                return  _("Use UserPreferences to change settings of the selected user account", formatted=False)
             else:
-                return _("Use UserPreferences to change your settings or create an account.")
+                return _("Use UserPreferences to change your settings or create an account.", formatted=False)
 
         if form.has_key('save'): # Save user profile
             if self.request.request_method != 'POST':
-                return _("Use UserPreferences to change your settings or create an account.")
-            theuser = self.request.get_user_from_form()
-                
+                return _("Use UserPreferences to change your settings or create an account.", formatted=False)
+            theuser = self.request.user
+
             if not 'name' in theuser.auth_attribs:
                 # Require non-empty name
                 theuser.name = form.get('name', [theuser.name])[0]
             if not theuser.name:
-                return _("Empty user name. Please enter a user name.")
+                return _("Empty user name. Please enter a user name.", formatted=False)
 
             # Don't allow users with invalid names
             if not user.isValidName(self.request, theuser.name):
                 return _("""Invalid user name {{{'%s'}}}.
 Name may contain any Unicode alpha numeric character, with optional one
-space between words. Group page name is not allowed.""") % wikiutil.escape(theuser.name)
+space between words. Group page name is not allowed.""", formatted=True, percent=True) % wikiutil.escape(theuser.name)
 
             # Is this an existing user trying to change information or a new user?
             # Name required to be unique. Check if name belong to another user.
             newuser = 1
             if user.getUserId(self.request, theuser.name):
                 if theuser.name != self.request.user.name:
-                    return _("This user name already belongs to somebody else.")
+                    return _("This user name already belongs to somebody else.", formatted=False)
                 else:
                     newuser = 0
 
             if not 'password' in theuser.auth_attribs:
-                # try to get the password and pw repeat
-                password = form.get('password', [''])[0]
-                password2 = form.get('password2',[''])[0]
+                # try to get the password and pw repeat                
+                password = form.get('password1', [''])[0]
+                password2 = form.get('password2', [''])[0]
 
                 # Check if password is given and matches with password repeat
                 if password != password2:
-                    return _("Passwords don't match!")
-                if not password and newuser:
-                    return _("Please specify a password!")
+                    return _("Passwords don't match!", formatted=False)
+                if not password:
+                    if newuser:
+                        return _("Please specify a password!", formatted=False)
+                else:
+                    pw_checker = self.request.cfg.password_checker
+                    if pw_checker:
+                        pw_error = pw_checker(theuser.name, password)
+                        if pw_error:
+                            return _("Password not acceptable: %s", formatted=False) % pw_error
+
                 # Encode password
                 if password and not password.startswith('{SHA}'):
                     try:
@@ -225,13 +255,13 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
 
             if not 'email' in theuser.auth_attribs:
                 # try to get the email
-                email = form.get('email', [theuser.email])[0]
+                email = wikiutil.clean_input(form.get('email', [theuser.email])[0])
                 theuser.email = email.strip()
 
             # Require email
             if not theuser.email:
                 return _("Please provide your email address. If you lose your"
-                         " login information, you can get it by email.")
+                         " login information, you can get it by email.", formatted=False)
 
             # Email should be unique - see also MoinMoin/script/accounts/moin_usercheck.py
             if theuser.email and self.request.cfg.user_email_unique:
@@ -241,22 +271,22 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                         continue
                     thisuser = user.User(self.request, uid, auth_method='userform:283')
                     if thisuser.email == theuser.email:
-                        return _("This email already belongs to somebody else.")
+                        return _("This email already belongs to somebody else.", formatted=False)
 
             if not 'aliasname' in theuser.auth_attribs:
                 # aliasname
-                theuser.aliasname = form.get('aliasname', [''])[0]
+                theuser.aliasname = wikiutil.clean_input(form.get('aliasname', [''])[0])
 
             # editor size
             theuser.edit_rows = util.web.getIntegerInput(self.request, 'edit_rows', theuser.edit_rows, 10, 60)
-                
+
             # try to get the editor
             theuser.editor_default = form.get('editor_default', [self.cfg.editor_default])[0]
             theuser.editor_ui = form.get('editor_ui', [self.cfg.editor_ui])[0]
 
             # time zone
             theuser.tz_offset = util.web.getIntegerInput(self.request, 'tz_offset', theuser.tz_offset, -84600, 84600)
-    
+
             # datetime format
             try:
                 dt_d_combined = UserSettings._date_formats.get(form['datetime_fmt'][0], '')
@@ -264,7 +294,7 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
             except (KeyError, ValueError):
                 theuser.datetime_fmt = '' # default
                 theuser.date_fmt = '' # default
-    
+
             # try to get the (optional) theme
             theme_name = form.get('theme_name', [self.cfg.theme_default])[0]
             if theme_name != theuser.theme_name:
@@ -277,10 +307,15 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                 theuser.theme_name = theme_name
                 if self.request.loadTheme(theuser.theme_name) > 0:
                     theme_name = wikiutil.escape(theme_name)
-                    return _("The theme '%(theme_name)s' could not be loaded!") % locals()
+                    return _("The theme '%(theme_name)s' could not be loaded!", formatted=False) % locals()
 
             # try to get the (optional) preferred language
             theuser.language = form.get('language', [''])[0]
+            if theuser.language == u'': # For language-statistics
+                from MoinMoin import i18n
+                theuser.real_language = i18n.get_browser_language(self.request)
+            else:
+                theuser.real_language = ''
 
             # I want to handle all inputs from user_form_fields, but
             # don't want to handle the cases that have already been coded
@@ -290,10 +325,10 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
             # handler for each form field, instead of stuffing them all in
             # one long and inextensible method.  That would allow for
             # plugins to provide methods to validate their fields as well.
-            already_handled = ['name', 'password', 'password2', 'email',
+            already_handled = ['name', 'password', 'password1', 'password2', 'email',
                                'aliasname', 'edit_rows', 'editor_default',
                                'editor_ui', 'tz_offset', 'datetime_fmt',
-                               'theme_name', 'language']
+                               'theme_name', 'language', 'real_language']
             for field in self.cfg.user_form_fields:
                 key = field[0]
                 if ((key in self.cfg.user_form_disable)
@@ -302,7 +337,7 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                 default = self.cfg.user_form_defaults[key]
                 value = form.get(key, [default])[0]
                 setattr(theuser, key, value)
-            
+
             # checkbox options
             if not newuser:
                 for key, label in self.cfg.user_checkbox_fields:
@@ -314,22 +349,36 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                             pass
                         else:
                             setattr(theuser, key, value)
-    
+
             # quicklinks for navibar
-            theuser.quicklinks = self.decodePageList('quicklinks')            
-            
+            theuser.quicklinks = self.decodePageList('quicklinks')
+
             # subscription for page change notification
             theuser.subscribed_pages = self.decodePageList('subscribed_pages')
-                    
+
             # save data
-            theuser.save()            
+            theuser.save()
             self.request.user = theuser
 
-            result = _("User preferences saved!")
+            result = _("User preferences saved!", formatted=False)
             if _debug:
                 result = result + util.dumpFormData(form)
             return result
 
+        # we keep those 2 separate so we maybe can use buttons later:
+        if form.has_key('disable_user') and self.request.user.isSuperUser():
+            user_name = form.get('name', [''])[0]
+            uid = user.getUserId(self.request, user_name)
+            theuser = user.User(self.request, uid)
+            theuser.disabled = 1
+            theuser.save()
+        if form.has_key('enable_user') and self.request.user.isSuperUser():
+            user_name = form.get('name', [''])[0]
+            uid = user.getUserId(self.request, user_name)
+            theuser = user.User(self.request, uid)
+            theuser.disabled = 0
+            theuser.save()
+
 
 #############################################################################
 ### Form Generation
@@ -339,10 +388,10 @@ class UserSettings:
     """ User login and settings management. """
 
     _date_formats = { # datetime_fmt & date_fmt
-        'iso':  '%Y-%m-%d %H:%M:%S & %Y-%m-%d',
-        'us':   '%m/%d/%Y %I:%M:%S %p & %m/%d/%Y',
+        'iso': '%Y-%m-%d %H:%M:%S & %Y-%m-%d',
+        'us': '%m/%d/%Y %I:%M:%S %p & %m/%d/%Y',
         'euro': '%d.%m.%Y %H:%M:%S & %d.%m.%Y',
-        'rfc':  '%a %b %d %H:%M:%S %Y & %a %b %d %Y',
+        'rfc': '%a %b %d %H:%M:%S %Y & %a %b %d %Y',
     }
 
     def __init__(self, request):
@@ -367,13 +416,13 @@ class UserSettings:
             options.append((
                 str(offset),
                 '%s [%s%s:%s]' % (
-                    time.strftime(self.cfg.datetime_fmt, timefuncs.tmtuple(t)),
+                    time.strftime(self.cfg.datetime_fmt, util.timefuncs.tmtuple(t)),
                     "+-"[offset < 0],
-                    string.zfill("%d" % (abs(offset) / 3600), 2),
-                    string.zfill("%d" % (abs(offset) % 3600 / 60), 2),
+                    "%02d" % (abs(offset) / 3600),
+                    "%02d" % (abs(offset) % 3600 / 60),
                 ),
             ))
- 
+
         return util.web.makeSelection('tz_offset', options, str(tz))
 
 
@@ -387,7 +436,7 @@ class UserSettings:
                     if v == dt_d_combined][0]
         except IndexError:
             selected = ''
-        options = [('', _('Default'))] + self._date_formats.items()
+        options = [('', _('Default', formatted=False))] + self._date_formats.items()
 
         return util.web.makeSelection('datetime_fmt', options, selected)
 
@@ -395,56 +444,55 @@ class UserSettings:
     def _lang_select(self):
         """ Create language selection. """
         from MoinMoin import i18n
-        from MoinMoin.i18n import NAME
         _ = self._
         cur_lang = self.request.user.valid and self.request.user.language or ''
         langs = i18n.wikiLanguages().items()
-        langs.sort(lambda x,y,NAME=NAME: cmp(x[1][NAME], y[1][NAME]))
+        langs.sort(lambda x, y: cmp(x[1]['x-language'], y[1]['x-language']))
         options = [('', _('<Browser setting>', formatted=False))]
         for lang in langs:
-            name = lang[1][NAME]
+            name = lang[1]['x-language']
             options.append((lang[0], name))
-                
+
         return util.web.makeSelection('language', options, cur_lang)
-  
+
     def _user_select(self):
         options = []
         users = user.getUserList(self.request)
         for uid in users:
-            name = user.User(self.request, id=uid).name # + '_' + uid # for debugging
+            name = user.User(self.request, id=uid).name # + '/' + uid # for debugging
             options.append((name, name))
         options.sort()
 
-        size = min(5, len(options))  
+        size = min(5, len(options))
         current_user = self.request.user.name
         return util.web.makeSelection('selected_user', options, current_user, size=size)
-            
+
     def _theme_select(self):
         """ Create theme selection. """
         cur_theme = self.request.user.valid and self.request.user.theme_name or self.cfg.theme_default
-        options = [("<default>", "<%s>" % self._("Default"))]
+        options = [("<default>", "<%s>" % self._("Default", formatted=False))]
         for theme in wikiutil.getPlugins('theme', self.request.cfg):
             options.append((theme, theme))
-                
+
         return util.web.makeSelection('theme_name', options, cur_theme)
-  
+
     def _editor_default_select(self):
         """ Create editor selection. """
         editor_default = self.request.user.valid and self.request.user.editor_default or self.cfg.editor_default
-        options = [("<default>", "<%s>" % self._("Default"))]
-        for editor in ['text','gui',]:
+        options = [("<default>", "<%s>" % self._("Default", formatted=False))]
+        for editor in ['text', 'gui', ]:
             options.append((editor, editor))
         return util.web.makeSelection('editor_default', options, editor_default)
 
     def _editor_ui_select(self):
         """ Create editor selection. """
         editor_ui = self.request.user.valid and self.request.user.editor_ui or self.cfg.editor_ui
-        options = [("<default>", "<%s>" % self._("Default")),
-                   ("theonepreferred", self._("the one preferred")),
-                   ("freechoice", self._("free choice")),
+        options = [("<default>", "<%s>" % self._("Default", formatted=False)),
+                   ("theonepreferred", self._("the one preferred", formatted=False)),
+                   ("freechoice", self._("free choice", formatted=False)),
                   ]
         return util.web.makeSelection('editor_ui', options, editor_ui)
-                
+
     def make_form(self):
         """ Create the FORM, and the TABLE with the input fields
         """
@@ -477,21 +525,23 @@ class UserSettings:
         _ = self._
         self.make_form()
 
-        if self.request.user.isSuperUser():
-            ticket = wikiutil.createTicket()
-            self.make_row(_('Select User'), [self._user_select()])
+        if (self.request.user.isSuperUser() or
+            (not self.request._setuid_real_user is None and
+             self.request._setuid_real_user.isSuperUser())):
+            ticket = wikiutil.createTicket(self.request, action='userform')
+            self.make_row(_('Select User', formatted=False), [self._user_select()])
             self._form.append(html.INPUT(type="hidden", name="ticket", value="%s" % ticket))
-            buttons = [("select_user", _('Select User'))]
+            buttons = [("select_user", _('Select User', formatted=False))]
             button_cell = []
             for name, label in buttons:
-                 button_cell.extend([
-                     html.INPUT(type="submit", name=name, value=label),
-                     ' ',
-                 ])
+                button_cell.extend([
+                    html.INPUT(type="submit", name=name, value=label),
+                    ' ',
+                ])
             self.make_row('', button_cell)
-            
+
         if self.request.user.valid and not create_only:
-            buttons = [('save', _('Save')), ('cancel', _('Cancel')), ]
+            buttons = [('save', _('Save', formatted=False)), ('cancel', _('Cancel', formatted=False)), ]
             uf_remove = self.cfg.user_form_remove
             uf_disable = self.cfg.user_form_disable
             for attr in self.request.user.auth_attribs:
@@ -504,38 +554,44 @@ class UserSettings:
                 default = self.cfg.user_form_defaults[key]
                 if not key in uf_remove:
                     if key in uf_disable:
-                        self.make_row(_(label),
-                                  [ html.INPUT(type=type, size=length, name=key, disabled="disabled",
-                                    value=getattr(self.request.user, key)), ' ', _(textafter), ])
+                        self.make_row(_(label, formatted=False),
+                                  [html.INPUT(type=type, size=length, name=key, disabled="disabled",
+                                   value=getattr(self.request.user, key)), ' ', _(textafter, formatted=False), ])
                     else:
-                        self.make_row(_(label),
-                                  [ html.INPUT(type=type, size=length, name=key, value=getattr(self.request.user, key)), ' ', _(textafter), ])
+                        # replace 'password' with 'password1' in the preferences form so browsers
+                        # won't fill in the users's password automatically
+                        if key == 'password':
+                            name = 'password1'
+                        else:
+                            name = key                            
+                        self.make_row(_(label, formatted=False),
+                                  [html.INPUT(type=type, size=length, name=name, value=getattr(self.request.user, key)), ' ', _(textafter, formatted=False), ])
 
             if not self.cfg.theme_force and not "theme_name" in self.cfg.user_form_remove:
-                self.make_row(_('Preferred theme'), [self._theme_select()])
+                self.make_row(_('Preferred theme', formatted=False), [self._theme_select()])
 
             if not self.cfg.editor_force:
                 if not "editor_default" in self.cfg.user_form_remove:
-                    self.make_row(_('Editor Preference'), [self._editor_default_select()])
+                    self.make_row(_('Editor Preference', formatted=False), [self._editor_default_select()])
                 if not "editor_ui" in self.cfg.user_form_remove:
-                    self.make_row(_('Editor shown on UI'), [self._editor_ui_select()])
+                    self.make_row(_('Editor shown on UI', formatted=False), [self._editor_ui_select()])
 
             if not "tz_offset" in self.cfg.user_form_remove:
-                self.make_row(_('Time zone'), [
-                    _('Your time is'), ' ',
+                self.make_row(_('Time zone', formatted=False), [
+                    _('Your time is', formatted=False), ' ',
                     self._tz_select(),
                     html.BR(),
-                    _('Server time is'), ' ',
-                    time.strftime(self.cfg.datetime_fmt, timefuncs.tmtuple()),
+                    _('Server time is', formatted=False), ' ',
+                    time.strftime(self.cfg.datetime_fmt, util.timefuncs.tmtuple()),
                     ' (UTC)',
                 ])
 
             if not "datetime_fmt" in self.cfg.user_form_remove:
-                self.make_row(_('Date format'), [self._dtfmt_select()])
+                self.make_row(_('Date format', formatted=False), [self._dtfmt_select()])
 
             if not "language" in self.cfg.user_form_remove:
-                self.make_row(_('Preferred language'), [self._lang_select()])
-            
+                self.make_row(_('Preferred language', formatted=False), [self._lang_select()])
+
             # boolean user options
             bool_options = []
             checkbox_fields = self.cfg.user_checkbox_fields
@@ -549,9 +605,9 @@ class UserSettings:
                             disabled=key in self.cfg.user_checkbox_disable and True or None),
                         ' ', label(_), html.BR(),
                     ])
-            self.make_row(_('General options'), bool_options, valign="top")
+            self.make_row(_('General options', formatted=False), bool_options, valign="top")
 
-            self.make_row(_('Quick links'), [
+            self.make_row(_('Quick links', formatted=False), [
                 html.TEXTAREA(name="quicklinks", rows="6", cols="50")
                     .append('\n'.join(self.request.user.getQuickLinks())),
             ], valign="top")
@@ -569,11 +625,11 @@ class UserSettings:
                         html.BR(),
                         html.SMALL(Class="warning").append(
                             _("This list does not work, unless you have"
-                              " entered a valid email address!")
+                              " entered a valid email address!", formatted=False)
                         )]
-                
+
                 self.make_row(
-                    html.Raw(_('Subscribed wiki pages (one regex per line)')),
+                    html.Raw(_('Subscribed wiki pages (one regex per line)', formatted=False)),
                     [html.TEXTAREA(name="subscribed_pages", rows="6", cols="50").append(
                         '\n'.join(notifylist)),
                     ] + warning,
@@ -585,24 +641,29 @@ class UserSettings:
                 # IMPORTANT: login should be first to be the default
                 # button when a user hits ENTER.
                 #('login', _('Login')),  # we now have a Login macro
-                ('create', _('Create Profile')),
-                ('cancel', _('Cancel')),
+                ('create', _('Create Profile', formatted=False)),
+                ('cancel', _('Cancel', formatted=False)),
             ]
             for key, label, type, length, textafter in self.cfg.user_form_fields:
                 if key in ('name', 'password', 'password2', 'email'):
-                    self.make_row(_(label),
-                              [ html.INPUT(type=type, size=length, name=key,
-                                           value=''),
-                                ' ', _(textafter), ])
+                    self.make_row(_(label, formatted=False),
+                              [html.INPUT(type=type, size=length, name=key,
+                                          value=''),
+                               ' ', _(textafter, formatted=False), ])
+
+        from MoinMoin.security.textcha import TextCha
+        textcha_rendered = TextCha(self.request).render()
+        if textcha_rendered:
+            self.make_row(_('TextCha (required)', formatted=False), textcha_rendered)
 
         if self.cfg.mail_enabled:
-            buttons.append(("account_sendmail", _('Mail me my account data')))
+            buttons.append(("account_sendmail", _('Mail me my account data', formatted=False)))
 
         if create_only:
-            buttons = [("create_only", _('Create Profile'))]
+            buttons = [("create_only", _('Create Profile', formatted=False))]
             if self.cfg.mail_enabled:
                 buttons.append(("create_and_mail", "%s + %s" %
-                                (_('Create Profile'), _('Email'))))
+                                (_('Create Profile', formatted=False), _('Email', formatted=False))))
 
         # Add buttons
         button_cell = []
@@ -648,28 +709,30 @@ class Login:
         sn = request.getScriptname()
         pi = request.getPathinfo()
         action = u"%s%s" % (sn, pi)
-        userprefslink = wikiutil.getSysPage(request, "UserPreferences").link_to(request)
-        hint = _("To create an account or recover a lost password, see the %(userprefslink)s page.") % {
-               'userprefslink': userprefslink}
-        self._form = html.FORM(action=action)
+        userprefslink = wikiutil.getLocalizedPage(request, "UserPreferences").link_to(request, rel='nofollow')
+        sendmypasswordlink = wikiutil.getLocalizedPage(request, "SendMyPassword").link_to(request, rel='nofollow')
+        hint = _("To create an account, see the %(userprefslink)s page. To recover a lost password, go to %(sendmypasswordlink)s.", formatted=False) % {
+                 'userprefslink': userprefslink,
+                 'sendmypasswordlink': sendmypasswordlink}
+        self._form = html.FORM(action=action, name="loginform")
         self._table = html.TABLE(border="0")
 
         # Use the user interface language and direction
         lang_attr = request.theme.ui_lang_attr()
-        self._form.append(html.Raw('<div class="userprefs"%s>' % lang_attr))
+        self._form.append(html.Raw('<div class="userpref"%s>' % lang_attr))
 
         self._form.append(html.INPUT(type="hidden", name="action", value="login"))
         self._form.append(self._table)
         self._form.append(html.P().append(hint))
         self._form.append(html.Raw("</div>"))
 
-        self.make_row(_('Name'), [
+        self.make_row(_('Name', formatted=False), [
             html.INPUT(
                 type="text", size="32", name="name",
             ),
         ])
 
-        self.make_row(_('Password'), [
+        self.make_row(_('Password', formatted=False), [
             html.INPUT(
                 type="password", size="32", name="password",
             ),
@@ -677,7 +740,7 @@ class Login:
 
         self.make_row('', [
             html.INPUT(
-                type="submit", name='login', value=_('Login')
+                type="submit", name='login', value=_('Login', formatted=False)
             ),
         ])
 
@@ -699,10 +762,9 @@ def do_user_browser(request):
 
     data = TupleDataset()
     data.columns = [
-        #Column('id', label=('ID'), align='right'),
         Column('name', label=('Username')),
         Column('email', label=('Email')),
-        Column('action', label=_('Action')),
+        Column('action', label=_('Action', formatted=False)),
     ]
 
     # Iterate over users
@@ -713,19 +775,40 @@ def do_user_browser(request):
         if userhomepage.exists():
             namelink = userhomepage.link_to(request)
         else:
-            namelink = account.name
+            namelink = wikiutil.escape(account.name)
+
+        if account.disabled:
+            enable_disable_link = request.page.link_to(
+                                    request, text=_('Enable user', formatted=False),
+                                    querystr={"action":"userform",
+                                              "name": account.name,
+                                              "enable_user": "1",
+                                              "sysadm": "users", },
+                                    rel='nofollow')
+            namelink += " (%s)" % _("disabled", formatted=False)
+        else:
+            enable_disable_link = request.page.link_to(
+                                    request, text=_('Disable user', formatted=False),
+                                    querystr={"action":"userform",
+                                              "name": account.name,
+                                              "disable_user": "1",
+                                              "sysadm": "users", },
+                                    rel='nofollow')
+
+        mail_link = request.page.link_to(
+                        request, text=_('Mail account data', formatted=False),
+                        querystr={"action":"userform",
+                                  "email": account.email,
+                                  "account_sendmail": "1",
+                                  "sysadm": "users", },
+                        rel='nofollow')
 
         data.addRow((
-            #request.formatter.code(1) + uid + request.formatter.code(0),
             request.formatter.rawHTML(namelink),
             (request.formatter.url(1, 'mailto:' + account.email, css='mailto', do_escape=0) +
              request.formatter.text(account.email) +
              request.formatter.url(0)),
-            request.page.link_to(request, text=_('Mail me my account data'),
-                                 querystr= {"action":"userform",
-                                            "email": account.email,  
-                                            "account_sendmail": "1",
-                                            "sysadm": "users",})
+            mail_link + " - " + enable_disable_link
         ))
 
     if data:
@@ -737,4 +820,3 @@ def do_user_browser(request):
 
     # No data
     return ''
-
