@@ -15,17 +15,17 @@
 
     Note that there are more config options than you'll find in
     the version of this file that is installed by default; see
-    the module MoinMoin.multiconfig for a full list of names and their
+    the module MoinMoin.config.multiconfig for a full list of names and their
     default values.
 
-    Also, the URL http://moinmoin.wikiwikiweb.de/HelpOnConfiguration has
+    Also, the URL http://moinmo.in/HelpOnConfiguration has
     a list of config options.
 
     ** Please do not use this file for a wiki farm. Use the sample file 
     from the wikifarm directory instead! **
 """
 
-from MoinMoin.multiconfig import DefaultConfig
+from MoinMoin.config.multiconfig import DefaultConfig
 
 
 class Config(DefaultConfig):
@@ -38,7 +38,7 @@ class Config(DefaultConfig):
     # Wiki logo. You can use an image, text or both. [Unicode]
     # For no logo or text, use '' - the default is to show the sitename.
     # See also url_prefix setting below!
-    logo_string = u'<img src="/wiki/common/moinmoin.png" alt="MoinMoin Logo">'
+    logo_string = u'<img src="/moin_static162/common/moinmoin.png" alt="MoinMoin Logo">'
 
     # name of entry page / front page [Unicode], choose one of those:
 
@@ -74,11 +74,14 @@ class Config(DefaultConfig):
     # backup it.
     data_underlay_dir = './underlay/'
 
-    # Location of your STATIC files (css/png/js/...) - you must NOT use the
-    # same for invoking moin.cgi (or, in general, the moin code).
-    # url_prefix must be '/wiki' for Twisted and standalone servers.
-    # For CGI, it should match your Apache Alias setting.
-    url_prefix = '/wiki'
+    # The URL prefix we use to access the static stuff (img, css, js).
+    # NOT touching this is maybe the best way to handle this setting as moin
+    # uses a good internal default (something like '/moin_static162' for moin
+    # version 1.6.2).
+    # For Twisted and standalone server, the default will automatically work.
+    # For others, you should make a matching server config (e.g. an Apache
+    # Alias definition pointing to the directory with the static stuff).
+    #url_prefix_static = '/moin_static162'
 
 
     # Security ----------------------------------------------------------
@@ -92,9 +95,14 @@ class Config(DefaultConfig):
     # All acl_rights_xxx options must use unicode [Unicode]
     #acl_rights_before = u"YourName:read,write,delete,revert,admin"
 
+    # The default (ENABLED) password_checker will keep users from choosing too
+    # short or too easy passwords. If you don't like this and your site has
+    # rather low security requirements, feel free to DISABLE the checker by:
+    #password_checker = None # None means "don't do any password strength checks"
+
     # Link spam protection for public wikis (Uncomment to enable)
     # Needs a reliable internet connection.
-    #from MoinMoin.util.antispam import SecurityPolicy
+    #from MoinMoin.security.antispam import SecurityPolicy
 
 
     # Mail --------------------------------------------------------------
@@ -134,7 +142,7 @@ class Config(DefaultConfig):
 
     # Language options --------------------------------------------------
 
-    # See http://moinmoin.wikiwikiweb.de/ConfigMarket for configuration in 
+    # See http://moinmo.in/ConfigMarket for configuration in 
     # YOUR language that other people contributed.
 
     # The main wiki language, set the direction of the wiki pages
