@@ -14,10 +14,10 @@
 
     Note that there are more config options than you'll find in
     the version of this file that is installed by default; see
-    the module MoinMoin.multiconfig for a full list of names and their
+    the module MoinMoin.config.multiconfig for a full list of names and their
     default values.
 
-    Also, the URL http://moinmoin.wikiwikiweb.de/HelpOnConfiguration has
+    Also, the URL http://moinmo.in/HelpOnConfiguration has
     a list of config options.
 """
 
@@ -42,14 +42,14 @@
 wikis = [
     # Standalone server needs the port e.g. localhost:8000
     # Twisted server can now use the port, too.
-    
+
     # wikiname,     url regular expression (no protocol)
     # ---------------------------------------------------------------
-    ("mywiki",  r".*"),   # this is ok for a single wiki
+    ("mywiki", r".*"),   # this is ok for a single wiki
 
     # for multiple wikis, do something like this:
-    #("moinmoin",    r"^moinmoin.wikiwikiweb.de/.*$"),
-    #("moinmaster",  r"^moinmaster.wikiwikiweb.de/.*$"),
+    #("moinmoin",    r"^moinmo.in/.*$"),
+    #("moinmaster",  r"^master.moinmo.in/.*$"),
 ]
 
 
@@ -65,7 +65,7 @@ wikis = [
 # this is to get everything to sane defaults, so we need to change only what
 # we like to have different:
 
-from MoinMoin.multiconfig import DefaultConfig
+from MoinMoin.config.multiconfig import DefaultConfig
 
 # Now we subclass this DefaultConfig. This means that we inherit every setting
 # from the DefaultConfig, except those we explicitely define different.
@@ -91,9 +91,14 @@ class FarmConfig(DefaultConfig):
     # backup it.
     data_underlay_dir = './underlay/'
 
-    # This must be '/wiki' for twisted and standalone. For CGI, it should
-    # match your Apache Alias setting.
-    url_prefix = '/wiki'
+    # The URL prefix we use to access the static stuff (img, css, js).
+    # NOT touching this is maybe the best way to handle this setting as moin
+    # uses a good internal default (something like '/moin_static162' for moin
+    # version 1.6.2).
+    # For Twisted and standalone server, the default will automatically work.
+    # For others, you should make a matching server config (e.g. an Apache
+    # Alias definition pointing to the directory with the static stuff).
+    #url_prefix_static = '/moin_static162'
 
 
     # Security ----------------------------------------------------------
@@ -109,7 +114,7 @@ class FarmConfig(DefaultConfig):
 
     # Link spam protection for public wikis (uncomment to enable).
     # Needs a reliable internet connection.
-    #from MoinMoin.util.antispam import SecurityPolicy
+    #from MoinMoin.security.antispam import SecurityPolicy
 
 
     # Mail --------------------------------------------------------------
@@ -149,7 +154,7 @@ class FarmConfig(DefaultConfig):
 
     # Language options --------------------------------------------------
 
-    # See http://moinmoin.wikiwikiweb.de/ConfigMarket for configuration in 
+    # See http://moinmo.in/ConfigMarket for configuration in 
     # YOUR language that other people contributed.
 
     # The main wiki language, set the direction of the wiki pages
